@@ -36,6 +36,7 @@
 // Copyright (C) 2024, 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 // Copyright (C) 2024 Fernando Herrera <fherrera@onirica.com>
 // Copyright (C) 2024, 2025 Nelson Benítez León <nbenitezl@gmail.com>
+// Copyright (C) 2025 Arnav V <arnav0872@gmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -54,6 +55,7 @@
 #include "poppler-config.h"
 #include "poppler_private_export.h"
 #include "Object.h"
+#include "goo/GooCheckedOps.h"
 
 class GooFile;
 class BaseStream;
@@ -126,8 +128,8 @@ public:
     // Get kind of stream.
     virtual StreamKind getKind() const = 0;
 
-    // Reset stream to beginning. Returns 'false' if stream was found to be invalid, 'true' otherwise.
-    [[nodiscard]] virtual bool reset() = 0;
+    // Rewind stream to beginning. Returns 'false' if stream was found to be invalid, 'true' otherwise.
+    [[nodiscard]] virtual bool rewind() = 0;
 
     // Close down the stream.
     virtual void close();
@@ -153,7 +155,7 @@ public:
     {
         unsigned char readBuf[4096];
         int readChars;
-        if (!reset()) {
+        if (!rewind()) {
             s.clear();
             return;
         }
@@ -173,7 +175,7 @@ public:
         int length = 0;
         int charsToRead = initialSize;
         bool continueReading = true;
-        if (!reset()) {
+        if (!rewind()) {
             return {};
         }
         while (continueReading && (readChars = doGetChars(charsToRead, buf.data() + length)) != 0) {
@@ -216,10 +218,10 @@ public:
     // Get next char directly from stream source, without filtering it
     virtual int getUnfilteredChar() = 0;
 
-    // Resets the stream without reading anything (even not the headers)
+    // Rewinds the stream without reading anything (even not the headers)
     // WARNING: Reading the stream with something else than getUnfilteredChar
-    // may lead to unexcepted behaviour until you call reset ()
-    [[nodiscard]] virtual bool unfilteredReset() = 0;
+    // may lead to unexcepted behaviour until you call rewind ()
+    [[nodiscard]] virtual bool unfilteredRewind() = 0;
 
     // Get next line from stream.
     virtual char *getLine(char *buf, int size);
@@ -391,7 +393,7 @@ public:
     BaseSeekInputStream(Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA);
     ~BaseSeekInputStream() override;
     StreamKind getKind() const override { return strWeird; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     void close() override;
     int getChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
     int lookChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
@@ -401,7 +403,7 @@ public:
     void moveStart(Goffset delta) override;
 
     int getUnfilteredChar() override { return getChar(); }
-    [[nodiscard]] bool unfilteredReset() override { return reset(); }
+    [[nodiscard]] bool unfilteredRewind() override { return rewind(); }
 
 protected:
     Goffset start;
@@ -447,7 +449,7 @@ public:
     Stream *getNextStream() const override { return str; }
 
     int getUnfilteredChar() override { return str->getUnfilteredChar(); }
-    [[nodiscard]] bool unfilteredReset() override { return str->unfilteredReset(); }
+    [[nodiscard]] bool unfilteredRewind() override { return str->unfilteredRewind(); }
 
 protected:
     Stream *str;
@@ -470,10 +472,10 @@ public:
     ImageStream(const ImageStream &) = delete;
     ImageStream &operator=(const ImageStream &other) = delete;
 
-    // Reset the stream.
-    [[nodiscard]] bool reset();
+    // Rewind the stream.
+    [[nodiscard]] bool rewind();
 
-    // Close the stream previously reset
+    // Close the stream previously rewind
     void close();
 
     // Gets the next pixel from the stream.  <pix> should be able to hold
@@ -541,7 +543,7 @@ private:
 // FileStream
 //------------------------------------------------------------------------
 
-#define fileStreamBufSize 256
+#define fileStreamBufSize 16384
 
 class POPPLER_PRIVATE_EXPORT FileStream : public BaseStream
 {
@@ -551,7 +553,7 @@ public:
     BaseStream *copy() override;
     std::unique_ptr<Stream> makeSubStream(Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA) override;
     StreamKind getKind() const override { return strFile; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     void close() override;
     int getChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
     int lookChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
@@ -561,7 +563,7 @@ public:
     void moveStart(Goffset delta) override;
 
     int getUnfilteredChar() override { return getChar(); }
-    [[nodiscard]] bool unfilteredReset() override { return reset(); }
+    [[nodiscard]] bool unfilteredRewind() override { return rewind(); }
 
     bool getNeedsEncryptionOnSave() const { return needsEncryptionOnSave; }
     void setNeedsEncryptionOnSave(bool needsEncryptionOnSaveA) { needsEncryptionOnSave = needsEncryptionOnSaveA; }
@@ -615,12 +617,12 @@ private:
 class POPPLER_PRIVATE_EXPORT CachedFileStream : public BaseStream
 {
 public:
-    CachedFileStream(CachedFile *ccA, Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA);
+    CachedFileStream(std::shared_ptr<CachedFile> ccA, Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA);
     ~CachedFileStream() override;
     BaseStream *copy() override;
     std::unique_ptr<Stream> makeSubStream(Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA) override;
     StreamKind getKind() const override { return strCachedFile; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     void close() override;
     int getChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
     int lookChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
@@ -630,12 +632,12 @@ public:
     void moveStart(Goffset delta) override;
 
     int getUnfilteredChar() override { return getChar(); }
-    [[nodiscard]] bool unfilteredReset() override { return reset(); }
+    [[nodiscard]] bool unfilteredRewind() override { return rewind(); }
 
 private:
     bool fillBuf();
 
-    CachedFile *cc;
+    std::shared_ptr<CachedFile> cc;
     Goffset start;
     bool limited;
     char buf[cachedStreamBufSize];
@@ -679,7 +681,7 @@ public:
 
     StreamKind getKind() const override { return strWeird; }
 
-    [[nodiscard]] bool reset() override
+    [[nodiscard]] bool rewind() override
     {
         bufPtr = buf + start;
         return true;
@@ -721,7 +723,7 @@ public:
 
     int getUnfilteredChar() override { return getChar(); }
 
-    bool unfilteredReset() override { return reset(); }
+    bool unfilteredRewind() override { return rewind(); }
 
 protected:
     T *buf;
@@ -785,6 +787,16 @@ public:
 // EmbedStream is deleted, reads from the base stream will proceed where
 // the BaseStream left off.  Note that this is very different behavior
 // that creating a new FileStream (using makeSubStream).
+//
+// When an EmbedStream has been created with reusable set to true, it keeps a copy of
+// what is read in memory. This allows rewinding it (which is generally not possible).
+// After rewind whatever was recorded will be returned. When reaching the end of recorded
+// data and returning EOF once, the stream will go back to reading from its underlying stream.
+// Thw fact that an extra EOF is added when reaching the end of recorded data makes this
+// mostly only useful if you have read the full stream and want to go back to the beginning
+// to read it again.
+//
+// EmbedStream decreases it's length every time a character is read
 //------------------------------------------------------------------------
 
 class POPPLER_PRIVATE_EXPORT EmbedStream : public BaseStream
@@ -795,7 +807,7 @@ public:
     BaseStream *copy() override;
     std::unique_ptr<Stream> makeSubStream(Goffset start, bool limitedA, Goffset lengthA, Object &&dictA) override;
     StreamKind getKind() const override { return str->getKind(); }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override;
     int lookChar() override;
     Goffset getPos() override;
@@ -804,25 +816,21 @@ public:
     void moveStart(Goffset delta) override;
 
     int getUnfilteredChar() override { return str->getUnfilteredChar(); }
-    [[nodiscard]] bool unfilteredReset() override { return str->unfilteredReset(); }
-
-    void rewind();
-    void restore();
+    [[nodiscard]] bool unfilteredRewind() override { return str->unfilteredRewind(); }
 
 private:
     bool hasGetChars() override { return true; }
     int getChars(int nChars, unsigned char *buffer) override;
 
     Stream *str;
-    bool limited;
-    bool reusable;
-    bool record;
+    const bool limited;
+    const bool reusable;
     bool replay;
     unsigned char *bufData;
     long bufMax;
     long bufLen;
     long bufPos;
-    Goffset start;
+    const Goffset initialLength;
 };
 
 //------------------------------------------------------------------------
@@ -835,7 +843,7 @@ public:
     explicit ASCIIHexStream(Stream *strA);
     ~ASCIIHexStream() override;
     StreamKind getKind() const override { return strASCIIHex; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override
     {
         int c = lookChar();
@@ -861,7 +869,7 @@ public:
     explicit ASCII85Stream(Stream *strA);
     ~ASCII85Stream() override;
     StreamKind getKind() const override { return strASCII85; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override
     {
         int ch = lookChar();
@@ -889,7 +897,7 @@ public:
     LZWStream(Stream *strA, int predictor, int columns, int colors, int bits, int earlyA);
     ~LZWStream() override;
     StreamKind getKind() const override { return strLZW; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override;
     int lookChar() override;
     int getRawChar() override;
@@ -949,7 +957,7 @@ public:
     explicit RunLengthStream(Stream *strA);
     ~RunLengthStream() override;
     StreamKind getKind() const override { return strRunLength; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
     int lookChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
     std::optional<std::string> getPSFilter(int psLevel, const char *indent) override;
@@ -979,7 +987,7 @@ public:
     CCITTFaxStream(Stream *strA, int encodingA, bool endOfLineA, bool byteAlignA, int columnsA, int rowsA, bool endOfBlockA, bool blackA, int damagedRowsBeforeErrorA);
     ~CCITTFaxStream() override;
     StreamKind getKind() const override { return strCCITTFax; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override
     {
         int c = lookChar();
@@ -990,7 +998,7 @@ public:
     std::optional<std::string> getPSFilter(int psLevel, const char *indent) override;
     bool isBinary(bool last = true) const override;
 
-    [[nodiscard]] bool unfilteredReset() override;
+    [[nodiscard]] bool unfilteredRewind() override;
 
     int getEncoding() { return encoding; }
     bool getEndOfLine() { return endOfLine; }
@@ -1001,7 +1009,7 @@ public:
     int getDamagedRowsBeforeError() { return damagedRowsBeforeError; }
 
 private:
-    [[nodiscard]] bool ccittReset(bool unfiltered);
+    [[nodiscard]] bool ccittRewind(bool unfiltered);
     int encoding; // 'K' parameter
     bool endOfLine; // 'EndOfLine' parameter
     bool byteAlign; // 'EncodedByteAlign' parameter
@@ -1036,7 +1044,7 @@ private:
     }
 };
 
-#ifndef ENABLE_LIBJPEG
+#if !ENABLE_LIBJPEG
 //------------------------------------------------------------------------
 // DCTStream
 //------------------------------------------------------------------------
@@ -1076,17 +1084,17 @@ public:
     DCTStream(Stream *strA, int colorXformA, Dict *dict, int recursion);
     ~DCTStream() override;
     StreamKind getKind() const override { return strDCT; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     void close() override;
     int getChar() override;
     int lookChar() override;
     std::optional<std::string> getPSFilter(int psLevel, const char *indent) override;
     bool isBinary(bool last = true) const override;
 
-    [[nodiscard]] bool unfilteredReset() override;
+    [[nodiscard]] bool unfilteredRewind() override;
 
 private:
-    [[nodiscard]] bool dctReset(bool unfiltered);
+    [[nodiscard]] bool dctRewind(bool unfiltered);
     bool progressive; // set if in progressive mode
     bool interleaved; // set if in interleaved mode
     int width, height; // image size
@@ -1142,7 +1150,7 @@ private:
 
 #endif
 
-#ifndef ENABLE_ZLIB_UNCOMPRESS
+#if !ENABLE_ZLIB_UNCOMPRESS
 //------------------------------------------------------------------------
 // FlateStream
 //------------------------------------------------------------------------
@@ -1180,17 +1188,17 @@ public:
     FlateStream(Stream *strA, int predictor, int columns, int colors, int bits);
     ~FlateStream() override;
     StreamKind getKind() const override { return strFlate; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override;
     int lookChar() override;
     int getRawChar() override;
     void getRawChars(int nChars, int *buffer) override;
     std::optional<std::string> getPSFilter(int psLevel, const char *indent) override;
     bool isBinary(bool last = true) const override;
-    [[nodiscard]] bool unfilteredReset() override;
+    [[nodiscard]] bool unfilteredRewind() override;
 
 private:
-    [[nodiscard]] bool flateReset(bool unfiltered);
+    [[nodiscard]] bool flateRewind(bool unfiltered);
     inline int doGetRawChar()
     {
         int c;
@@ -1256,7 +1264,7 @@ public:
     explicit EOFStream(Stream *strA);
     ~EOFStream() override;
     StreamKind getKind() const override { return strWeird; }
-    [[nodiscard]] bool reset() override { return true; }
+    [[nodiscard]] bool rewind() override { return true; }
     int getChar() override { return EOF; }
     int lookChar() override { return EOF; }
     std::optional<std::string> getPSFilter(int /*psLevel*/, const char * /*indent*/) override { return {}; }
@@ -1273,10 +1281,10 @@ public:
     BufStream(Stream *strA, int bufSizeA);
     ~BufStream() override;
     StreamKind getKind() const override { return strWeird; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override;
     int lookChar() override;
-    std::optional<std::string> getPSFilter(int psLevel, const char *indent) override { return {}; }
+    std::optional<std::string> getPSFilter(int /*psLevel*/, const char * /*indent*/) override { return {}; }
     bool isBinary(bool last = true) const override;
 
     int lookChar(int idx);
@@ -1296,7 +1304,7 @@ public:
     FixedLengthEncoder(Stream *strA, int lengthA);
     ~FixedLengthEncoder() override;
     StreamKind getKind() const override { return strWeird; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override;
     int lookChar() override;
     std::optional<std::string> getPSFilter(int /*psLevel*/, const char * /*indent*/) override { return {}; }
@@ -1318,7 +1326,7 @@ public:
     explicit ASCIIHexEncoder(Stream *strA);
     ~ASCIIHexEncoder() override;
     StreamKind getKind() const override { return strWeird; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
     int lookChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
     std::optional<std::string> getPSFilter(int /*psLevel*/, const char * /*indent*/) override { return {}; }
@@ -1345,7 +1353,7 @@ public:
     explicit ASCII85Encoder(Stream *strA);
     ~ASCII85Encoder() override;
     StreamKind getKind() const override { return strWeird; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
     int lookChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
     std::optional<std::string> getPSFilter(int /*psLevel*/, const char * /*indent*/) override { return {}; }
@@ -1372,7 +1380,7 @@ public:
     explicit RunLengthEncoder(Stream *strA);
     ~RunLengthEncoder() override;
     StreamKind getKind() const override { return strWeird; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
     int lookChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
     std::optional<std::string> getPSFilter(int /*psLevel*/, const char * /*indent*/) override { return {}; }
@@ -1406,11 +1414,11 @@ public:
     explicit LZWEncoder(Stream *strA);
     ~LZWEncoder() override;
     StreamKind getKind() const override { return strWeird; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override;
     int lookChar() override;
-    std::optional<std::string> getPSFilter(int psLevel, const char *indent) override { return {}; }
-    bool isBinary(bool last = true) const override { return true; }
+    std::optional<std::string> getPSFilter(int /*psLevel*/, const char * /*indent*/) override { return {}; }
+    bool isBinary(bool /*last*/ = true) const override { return true; }
     bool isEncoder() const override { return true; }
 
 private:
@@ -1436,7 +1444,7 @@ public:
     explicit CMYKGrayEncoder(Stream *strA);
     ~CMYKGrayEncoder() override;
     StreamKind getKind() const override { return strWeird; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
     int lookChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
     std::optional<std::string> getPSFilter(int /*psLevel*/, const char * /*indent*/) override { return {}; }
@@ -1462,7 +1470,7 @@ public:
     explicit RGBGrayEncoder(Stream *strA);
     ~RGBGrayEncoder() override;
     StreamKind getKind() const override { return strWeird; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr++ & 0xff); }
     int lookChar() override { return (bufPtr >= bufEnd && !fillBuf()) ? EOF : (*bufPtr & 0xff); }
     std::optional<std::string> getPSFilter(int /*psLevel*/, const char * /*indent*/) override { return {}; }
@@ -1491,7 +1499,7 @@ public:
     explicit SplashBitmapCMYKEncoder(SplashBitmap *bitmapA);
     ~SplashBitmapCMYKEncoder() override;
     StreamKind getKind() const override { return strWeird; }
-    [[nodiscard]] bool reset() override;
+    [[nodiscard]] bool rewind() override;
     int getChar() override;
     int lookChar() override;
     std::optional<std::string> getPSFilter(int /*psLevel*/, const char * /*indent*/) override { return {}; }
@@ -1502,7 +1510,7 @@ public:
     bool isEncoder() const override { return false; }
 
     int getUnfilteredChar() override { return getChar(); }
-    [[nodiscard]] bool unfilteredReset() override { return reset(); }
+    [[nodiscard]] bool unfilteredRewind() override { return rewind(); }
 
     BaseStream *getBaseStream() override { return nullptr; }
     Stream *getUndecodedStream() override { return this; }
@@ -1529,10 +1537,10 @@ private:
 // Object Stream accessors.
 //------------------------------------------------------------------------
 
-inline bool Object::streamReset()
+inline bool Object::streamRewind()
 {
     OBJECT_TYPE_CHECK(objStream);
-    return stream->reset();
+    return stream->rewind();
 }
 
 inline void Object::streamClose()
